`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:57:36 12/11/2008 
// Design Name: 
// Module Name:    analyzer_adc 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module analyzer_adc(
								adcdry,					// ADC Data ready
								clk,					// System clock
								reset,					// System reset
								capture_st,				// Capture start
								adc_data,				// ADC Data
								sample_adr,				// Sample buffer address
								sample_data				// Sample buffer data
);

	////////////////////////////////////////////////////////////////////////////////
	//		Module Parameters
	////////////////////////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////////////////////////
	//		Module Interactions
	////////////////////////////////////////////////////////////////////////////////

	input						adcdry;					// ADC Data ready
	input						clk;					// System clock
	input						reset;					// System reset
	input						capture_st;				// Capture start
	input	[10:0]				adc_data;				// ADC Data
	input	[9:0]				sample_adr;				// Sample buffer address
	output	[10:0]				sample_data;			// Sample buffer data


	////////////////////////////////////////////////////////////////////////////////
	//		Register Declarations
	////////////////////////////////////////////////////////////////////////////////

	reg		[9:0]				capture_cnt;			// Sample buffer address counter for capture
	reg		[1:0]				capture_st_d;			// Capture busy delay
	reg							capture_busy;			// Capture busy


	////////////////////////////////////////////////////////////////////////////////
	//		Wire Declarations
	////////////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////////////
	//		 Module Instantiations
	////////////////////////////////////////////////////////////////////////////////

	dpram_11x1024  buf_adsample (
		.addra					(capture_cnt),
		.addrb					(sample_adr),
		.clka					(adcdry),
		.clkb					(clk),
		.dina					(adc_data),
		.doutb					(sample_data),
		.wea					(capture_busy)
	);


	////////////////////////////////////////////////////////////////////////////////
	//		  Module Logic
	////////////////////////////////////////////////////////////////////////////////

	//		 Capture start detection
	always@ (posedge adcdry) begin
		capture_st_d <= {capture_st_d[0],capture_st};
	end

	//		 Capture control
	always@ (posedge adcdry) begin
		if (!capture_busy) begin
			if (capture_st_d ==  2'b01) begin			// Up edge
				capture_busy <= 1;
				capture_cnt <= 0;
			end
		end else begin
			if (capture_cnt == 10'h3ff) begin
				capture_busy <= 0;
			end else begin
				capture_cnt <= capture_cnt + 1;
			end
		end
	end

endmodule
