`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:58:07 12/11/2008 
// Design Name: 
// Module Name:    analyzer_fft 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module analyzer_fft(
								clk,					// System clock
								reset,					// System reset
								sample_adr,				// Sample data buf address
								sample_data,			// Sample data buf data
								result_adr,				// Result data buf address
								result_data,			// Result data buf data
								capture_st				// Request for captue
);

	////////////////////////////////////////////////////////////////////////////////
	//		Module Parameters
	////////////////////////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////////////////////////
	//		Module Interactions
	////////////////////////////////////////////////////////////////////////////////

	input						clk;					// System clock
	input						reset;					// System reset
	output	[9:0]				sample_adr;				// Sample data buf address
	input	[10:0]				sample_data;			// Sample data buf data
	input	[8:0]				result_adr;				// Result data buf address
	output	[8:0]				result_data;			// Result data buf data
	output						capture_st;				// Request for captue


	////////////////////////////////////////////////////////////////////////////////
	//		Register Declarations
	////////////////////////////////////////////////////////////////////////////////

	reg							fwd_inv_we;				// FFT fwd_inv_we
	reg							start;					// FFT start
	reg		[1:0]				sequence;				// Sequence counter

	reg		[8:0]				xk_index_d [0:6];
	reg							dv_d [0:6];

	integer						I;

	////////////////////////////////////////////////////////////////////////////////
	//		Wire Declarations
	////////////////////////////////////////////////////////////////////////////////

	wire	[10:0]				window_data;

	wire						dv;						// FFT dv
	wire						edone;					// FFT edone
	wire						busy;					// FFT busy

	wire	[10:0]				xn_re;					// FFT xn_re
	wire	[21:0]				xk_re;					// FFT xk_re
	wire	[21:0]				xk_im;					// FFT xk_im
	wire	[9:0]				xk_index;				// FFT xk_index

	wire	[19:0]				xk_re_sq;
	wire	[19:0]				xk_im_sq;
	wire	[20:0]				xk_abs_sq;
	wire	[10:0]				xk_abs;

	wire	[8:0]				xk_abs_comp;


	////////////////////////////////////////////////////////////////////////////////
	//		 Module Instantiations
	////////////////////////////////////////////////////////////////////////////////

	fft_1024_11_unscale fft (
		.clk					(clk),					// input				clk
		.fwd_inv				(1),					// input				fwd_inv
		.fwd_inv_we				(fwd_inv_we),			// input				fwd_inv_we
		.start					(start),				// input				start
		.rfd					(),						// output				rfd
		.xn_re					(xn_re),				// input	[10 : 0] 	xn_re
		.xn_im					(0),					// input	[10 : 0] 	xn_im
		.xn_index				(sample_adr),			// output	[ 8 : 0] 	xn_index
		.unload					(edone),				// input				unload
		.dv						(dv),					// output				dv
		.done					(),						// output				done
		.busy					(busy),					// output				busy
		.edone					(edone),				// output				edone
		.xk_re					(xk_re),				// output	[10 : 0] 	xk_re
		.xk_im					(xk_im),				// output	[10 : 0] 	xk_im
		.xk_index				(xk_index)				// output	[ 8 : 0] 	xk_index
	);

	rom_11x1024 rom_window (
		.clka					(clk),
		.addra					(sample_adr),
		.douta					(window_data)
	);

	mul_11_11_11 mul_window (
		.clk					(clk),
		.a						(window_data),
		.b						(sample_data),
		.p						(xn_re)
	);

	mul_11_11_20 mul_xk_re_sq (
		.a						(xk_re[19:9]),
		.b						(xk_re[19:9]),
		.p						(xk_re_sq)
	);

	mul_11_11_20 mul_xk_im_sq (
		.a						(xk_im[19:9]),
		.b						(xk_im[19:9]),
		.p						(xk_im_sq)
	);

	sqrt_21 sqrt (
		.clk					(clk),
		.x_in					(xk_abs_sq),
		.x_out					(xk_abs)
	);

	dpram_9x512 buf_fftresult (
		.clka					(clk),
		.addra					(xk_index_d[6]),
		.dina					(xk_abs_comp),
		.wea					(dv_d[6]),
		.clkb					(clk),
		.addrb					(result_adr),
		.doutb					(result_data)
	);


	////////////////////////////////////////////////////////////////////////////////
	//		  Module Logic
	////////////////////////////////////////////////////////////////////////////////

	always@ (posedge clk) begin
		if (reset) begin								// Reset
			sequence 	<= 0;
			start		<= 0;
			fwd_inv_we 	<= 0;
		end else if (sequence == 0)	begin 				// Sequence 0 : Activate Start loading request
			start <= 1;
			fwd_inv_we <= 1;
			sequence <= sequence + 1;
		end else if (sequence == 1)	begin				// Sequence 1 : Negate Start loading request
			start <= 0;
			fwd_inv_we <= 0;
			sequence <= sequence + 1;
		end else if (sequence == 2) begin				// Sequencs 2 : Waiting for dv
			if (dv) begin
				sequence <= sequence + 1;
			end
		end else if (sequence == 3) begin				// Sequencs 3 : Waiting for end of unloading
			if (!dv) begin
				sequence <= 0;
			end
		end
	end

	always@ (posedge clk) begin
		dv_d[0] <= dv & (!xk_index[9]);
		xk_index_d[0] <= xk_index[8:0];

		for (I=0;I<=5;I=I+1) begin
			xk_index_d[I+1] <= xk_index_d[I];
			dv_d[I+1] <= dv_d[I];
		end
	end

	assign xk_abs_sq = xk_re_sq + xk_im_sq;
	assign capture_st = busy;

	function [8:0] pseudo_comp;
	input	[10:0] linear;
		if (linear & 11'b10000000000) begin
			pseudo_comp = {3'b101,linear[9:4]};
		end else if (linear & 11'b11000000000) begin
			pseudo_comp = {3'b100,linear[ 8:3]};
		end else if (linear & 11'b11100000000) begin
			pseudo_comp = {3'b011,linear[ 7:2]};
		end else if (linear & 11'b11110000000) begin
			pseudo_comp = {3'b010,linear[ 6:1]};
		end else if (linear & 11'b11111000000) begin
			pseudo_comp = {3'b001,linear[ 5:0]};
		end else begin
			pseudo_comp = {3'b000,linear[ 5:0]};
		end
	endfunction

	assign xk_abs_comp = pseudo_comp(xk_abs[10:0]);

endmodule
