`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:56:53 12/11/2008 
// Design Name: 
// Module Name:    analyzer_root 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module analyzer_root (
								CLK_48M,				// Master clock
								VGA_R,					// VGA Red Intensity
								VGA_G,					// VGA Green Intensity
								VGA_B,					// VGA Blue Intensity
								VGA_VS,					// VGA Virtical Sync
								VGA_HS,					// VGA Horizontal Sync
								ADC_CLK,				// ADC Operation Clock
								ADC_DATA,				// ADC Data
								ADC_DRY,				// ADC Data Ready
								ADC_OVR					// ADC Overflow
);


	////////////////////////////////////////////////////////////////////////////////
	//		Module Parameters
	////////////////////////////////////////////////////////////////////////////////

	parameter					RST_CNT_BITS = 20;


	////////////////////////////////////////////////////////////////////////////////
	//		Module Interactions
	////////////////////////////////////////////////////////////////////////////////

	input						CLK_48M;				// Master clock
	output	[3:0]				VGA_R;					// VGA Red Intensity
	output	[3:0]				VGA_G;					// VGA Green Intensity
	output	[3:0]				VGA_B;					// VGA Blue Intensity
	output						VGA_HS;					// VGA Virtical Sync
	output						VGA_VS;					// VGA Horizontal Sync
	output						ADC_CLK;				// ADC Operation Clock
	input	[10:0]				ADC_DATA;				// ADC Data
	input						ADC_DRY;				// ADC Data Ready
	input						ADC_OVR;				// ADC Overflow


	////////////////////////////////////////////////////////////////////////////////
	//		Register Declarations
	////////////////////////////////////////////////////////////////////////////////

	reg							clk_div;				// Master clok divider
	reg		[RST_CNT_BITS-1:0]	reset_cnt;				// Power-On reset counter


	////////////////////////////////////////////////////////////////////////////////
	//		Wire Declarations
	////////////////////////////////////////////////////////////////////////////////

	wire	[10:0]				sample_data;
	wire	[9:0]				sample_adr;
	wire	[8:0]				result_data;
	wire	[8:0]				result_adr;
	wire						capture_st;
	wire						clk;

	////////////////////////////////////////////////////////////////////////////////
	//		 Module Instantiations
	////////////////////////////////////////////////////////////////////////////////

	BUFG sysclk (
		.I						(clk_div),
		.O						(clk)
	);

	analyzer_video analyzer_video (
		.clk					(clk),
		.reset					(!reset_cnt[RST_CNT_BITS-1]),
		.vga_r					(VGA_R),
		.vga_g					(VGA_G),
		.vga_b					(VGA_B),
		.vga_hs					(VGA_HS),
		.vga_vs					(VGA_VS),
		.dispval				(result_data),
		.dispadr				(result_adr),
		.dispcolor				(ADC_OVR)
	);

	dfs_48_102p4 adcclkgen (
		.CLKIN_IN				(CLK_48M), 
		.RST_IN					(!reset_cnt[RST_CNT_BITS-1]), 
		.CLKFX_OUT				(ADC_CLK), 
		.LOCKED_OUT				()
    );

	analyzer_adc analyzer_adc (
		.adcdry					(ADC_DRY),
		.clk					(clk),
		.reset					(!reset_cnt[RST_CNT_BITS-1]),
		.adc_data				(ADC_DATA),
		.sample_adr				(sample_adr),
		.sample_data			(sample_data),
		.capture_st				(capture_st)
	);

	analyzer_fft analyzer_fft (
		.clk					(clk),
		.reset					(!reset_cnt[RST_CNT_BITS-1]),
		.sample_adr				(sample_adr),
		.sample_data			(sample_data),
		.result_adr				(result_adr),
		.result_data			(result_data),
		.capture_st				(capture_st)
	);

	////////////////////////////////////////////////////////////////////////////////
	//		  Module Logic
	////////////////////////////////////////////////////////////////////////////////

	//		  Master clock divider (1/2)
	always@ (posedge CLK_48M) begin
		clk_div <= !clk_div;
	end

	//		  Power-On reset counter
	always@ (posedge CLK_48M) begin
		if (!reset_cnt[RST_CNT_BITS-1]) begin
			reset_cnt <= reset_cnt + 1;
		end
	end

endmodule
