`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:58:41 12/11/2008 
// Design Name: 
// Module Name:    analyzer_video 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module analyzer_video(
								clk,					// System clock (24.000MHz)
								reset,					// System reset
								vga_r,					// VGA RED Intensity
								vga_g,					// VGA GREEN Intensity
								vga_b,					// VGA BLUE Intensity
								vga_hs,					// VGA Horizontal Sync
								vga_vs,					// VGA Virtical Sync
								dispval,				// Display Memory Value
								dispadr,				// Display Memory Address
								dispcolor				// Display color select
);


	////////////////////////////////////////////////////////////////////////////////
	//		Module Parameters
	////////////////////////////////////////////////////////////////////////////////

	parameter					HORIZONTAL_DISP_SIZE	= 640;
	parameter					VIRTICAL_DISP_SIZE		= 480;

	parameter					HORIZONTAL_TOTAL_PIXEL	= 800;
	parameter					VIRTICAL_TOTAL_PIXEL	= 521;

	parameter					HORIZONTAL_SYNC_START	= 656;
	parameter					HORIZONTAL_SYNC_END		= 752;

	parameter					VIRTICAL_SYNC_START		= 490;
	parameter					VIRTICAL_SYNC_END		= 492;


	////////////////////////////////////////////////////////////////////////////////
	//		Module Interactions
	////////////////////////////////////////////////////////////////////////////////

	input						clk;					// System clock (24.000MHz)
	input						reset;					// System reset
	output	[3:0]				vga_r;					// VGA RED Intensity
	output	[3:0]				vga_g;					// VGA GREEN Intensity
	output	[3:0]				vga_b;					// VGA BLUE Intensity
	output						vga_hs;					// VGA Horizontal Sync
	output						vga_vs;					// VGA Virtical Sync
	input	[8:0]				dispval;				// Display Memory Value
	output	[8:0]				dispadr;				// Display Memory Address
	input						dispcolor;				// Display color select


	////////////////////////////////////////////////////////////////////////////////
	//		Register Declarations
	////////////////////////////////////////////////////////////////////////////////

	reg		[9:0]				h_pixel_count;			// Horizontal pixel counter
	reg		[9:0]				v_pixel_count;			// Virtical pixel counter
	reg		[3:0]				h_marker_count;

	reg		[11:0]				vga_rgb;				// RGB intensity
	reg							vga_hs;					// Horizontal sync
	reg							vga_vs;					// Virtical sync

	reg		[8:0]				dispadr;				// Display memory address


	////////////////////////////////////////////////////////////////////////////////
	//		Wire Declarations
	////////////////////////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////////////////////////
	//		 Module Instantiations
	////////////////////////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////////////////////////
	//		  Module Logic
	////////////////////////////////////////////////////////////////////////////////

	//		  Pixel counter
	always @(posedge clk) begin
		if (reset) begin
			h_pixel_count <= 0;
			v_pixel_count <= 0;
			dispadr <= 0;
		end else if (h_pixel_count < HORIZONTAL_TOTAL_PIXEL) begin
			h_pixel_count <= h_pixel_count + 1;
			dispadr <= h_pixel_count[8:0] - 2;
		end else begin
			h_pixel_count <= 0;
			if (v_pixel_count < VIRTICAL_TOTAL_PIXEL) begin
				v_pixel_count <= v_pixel_count + 1;
			end else begin
				v_pixel_count <= 0;
			end
		end
	end

	//		  Draw logic
	always @(posedge clk) begin
		if (reset) begin
			vga_rgb <= 12'h000;
		end else if (h_pixel_count < HORIZONTAL_DISP_SIZE && v_pixel_count < VIRTICAL_DISP_SIZE) begin
			if( (v_pixel_count < 4) || (v_pixel_count > 475) ) begin
				vga_rgb <= 12'he61;
			end else begin
				if ( (h_pixel_count < 4) || (h_pixel_count > 635) ) begin
					vga_rgb <= 12'he61;
					h_marker_count <= 0;
				end else begin
					if ( (h_pixel_count >= 4 ) &&  (h_pixel_count < 516) && ((475 - v_pixel_count) < dispval) ) begin
						vga_rgb <= dispcolor ? 12'hf00 : 12'hee0;
					end else if ( (v_pixel_count == 92)  || (v_pixel_count == 156) || (v_pixel_count == 220) || (v_pixel_count == 284) || (v_pixel_count == 348) || 
								  (v_pixel_count == 412) || (v_pixel_count == 444) || (v_pixel_count == 460) || (v_pixel_count == 468) ) begin
						vga_rgb <= 12'h730; 
					end else if ( (h_pixel_count == 4)   || (h_pixel_count == 104) || (h_pixel_count == 204) || (h_pixel_count == 304) ||
								  (h_pixel_count == 404) || (h_pixel_count == 504) ) begin
						vga_rgb <= 12'h730; 
					end else if (h_marker_count == 0) begin
						vga_rgb <= 12'h006;
					end else begin
						vga_rgb <= 12'h004;
					end

					if (h_marker_count == 9) begin
						h_marker_count <= 0;
					end else begin
						h_marker_count <= h_marker_count + 1;
					end
				end
			end
		end	else if (h_pixel_count ==	HORIZONTAL_DISP_SIZE) begin
			vga_rgb <= 12'h000;
		end
	end

	//		  Sync pulse
	always @(posedge clk) begin
		if (reset) begin
			vga_hs <= 1'b0;
		end else if (h_pixel_count == HORIZONTAL_SYNC_START) begin
			vga_hs <= 1'b0;
		end else if(h_pixel_count ==  HORIZONTAL_SYNC_END) begin
			vga_hs <= 1'b1;
		end

		if (reset) begin
			vga_vs <= 1'b0;
		end else if (v_pixel_count ==  VIRTICAL_SYNC_START) begin
			vga_vs <= 1'b0;
		end else if(v_pixel_count == VIRTICAL_SYNC_END) begin
			vga_vs <= 1'b1;
		end
	end

	assign	vga_r	= vga_rgb[11:8];
	assign	vga_g	= vga_rgb[7:4];
	assign	vga_b	= vga_rgb[3:0];

endmodule
